/***************************************************************************
                           player.c
    description          : player handling
                           -------------------
    begin                : Fri March 30 2001, from rungame.c
    date                 : 04/08/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "aliens.h"
#include "bombs.h"
#include "data.h"
#include "shots.h"
#include "sound.h"
#include "structs.h" //global screen
#include "player.h"

#define PLAYER_SPEED    4
#define EXPLODE_TIME    30

static Sprite player;
static Sprite explosion;


/**
 *  player_init()
 *  initializes shot data
 *
 *  Returns:    -1 on error, otherwise count of Sprites.
 *
**/
int player_init(void)
{
	if ((player.image = LoadImage(DATAFILE("player.gif"), 0)) == NULL)
		return -1;
	player.x = (screen->w - player.image->w) / 2;
	player.y = (screen->h - player.image->h) - 1;
	if ((explosion.image = LoadImage(DATAFILE("explosion.gif"), 1)) == NULL)
		return -1;
	return 2;
}


/**
 *  player_free()
 *  frees shot data
 *
**/
void player_free(void)
{
	if (player.image)
		SDL_FreeSurface(player.image);
	if (explosion.image)
		SDL_FreeSurface(explosion.image);
}


int player_collide(void)
{
	int deaths;

	deaths = aliens_collide(&player, 0);
	deaths += bombs_collide(&player);
	if (deaths) {
		explosion.x = player.x;
		explosion.y = player.y;
		explosion.alive = EXPLODE_TIME;
		--player.alive;
	}
	return deaths;
}


void player_init_lives(void)
{
	player.alive = MAX_LIVES;
}


int player_get_lives(void)
{
	return player.alive;
}


int  player_add_life(void)
{
	if (player.alive < MAX_LIVES)
	{
		++player.alive;
		return player.alive;
	}
	return 0;
}


void player_right(void)
{
	player.x += PLAYER_SPEED;
	if (player.x >= screen->w - player.image->w)
		player.x = (screen->w -player.image->w) -1;
}

void player_left(void)
{
	player.x -= PLAYER_SPEED;
	if (player.x < 0)
		player.x = 0;
}


/**
 *  player_add_shot()
 *  adds a shot slot, if shot delay has timed out.
 *
**/
void player_add_shot(void)
{
	//Attempt a shot
	Sprite *shot = shots_add();
	if (shot)
	{
		shot->x = player.x +
		 (player.image->w - shot->image->w) / 2;
		shot->y = player.y -
				shot->image->h;
		shot->alive = 1;
		Mix_PlayChannel(SHOT_WAV,  //Make shot noise
				sounds[SHOT_WAV], 0);
	}
}


/**
 *  player_do()
 *  apply function to player.
 *
 *  @dofunc:    function to apply.
 *
**/
void player_do(SpriteFunc dofunc)
{
	if (player.alive)
		dofunc(&player);
	if (explosion.alive)
	{
		if (dofunc == sprite_ops[SPRITE_ERASE])
			--explosion.alive;
		dofunc(&explosion);
	}
}
